import * as fs from 'node:fs';
import { resolve } from 'node:path';
import { Middleware, storybook } from './index';
import { react, reactRefresh } from './presets/react';
import { tailwind } from './presets/tailwind';
import { reactQuery } from './presets/misc';

export function* checkEnv(): Generator<Middleware> {
  const rootDir = process.cwd();

  const pkgJsonPath = resolve(rootDir, 'package.json');
  const pkgJson = fs.existsSync(pkgJsonPath)
    ? JSON.parse(fs.readFileSync(pkgJsonPath, 'utf-8'))
    : {};

  const deps = new Set(
    Object.keys({
      ...pkgJson.dependencies,
      ...pkgJson.devDependencies,
      ...pkgJson.peerDependencies,
    }),
  );

  if (deps.has('react')) {
    yield react;
  }
  if (deps.has('@vitejs/plugin-react') && deps.has('eslint-plugin-react-refresh')) {
    yield reactRefresh;
  }
  if (deps.has('tailwindcss') && deps.has('eslint-plugin-tailwindcss')) {
    yield tailwind;
  }
  if (deps.has('storybook') && deps.has('eslint-plugin-storybook')) {
    yield storybook;
  }
  if (deps.has('@tanstack/react-query') && deps.has('@tanstack/eslint-plugin-query')) {
    yield reactQuery;
  }
}
